{{ Capacitive Load Cell Driver and Scales.
     (c) Copyright 2012 Phil Pilgrim
     See end of file for terms of use.
}}

CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  IF_PIN        = 0
  QF_PIN        = 1
  FB_PIN        = 9
  IN_PIN        = 8

OBJ

  pst   : "Parallax Serial Terminal"
  num   : "Simple_Numbers"
  u     : "umath"

VAR

  long  frequency, weight, stack[60]

PUB  start | tare

  pst.start(9600)
  frequency := 312500
  cognew(@quadrature, @frequency)
  cognew(weigh, @stack)
  ctra := %01001 << 26 | FB_PIN << 9 | IN_PIN
  dira[FB_PIN]~~
  frqb := 1
  waitcnt(cnt + clkfreq * 3)
  tare := weight
  repeat
    if (pst.rxcount)
      pst.charin
      waitcnt(cnt + clkfreq * 2)
      tare := weight
    pst.char(1)
    pst.dec(weight - tare)
    pst.char(11)

PUB weigh | t0, t1, p0, p1, p2, p3

    ctra := %10110 << 26 | FB_PIN << 9 | IF_PIN
    ctrb := %10110 << 26 | FB_PIN << 9 | QF_PIN
    frqa := 1
    frqb := 1
    repeat
      longfill(@p0, 0, 4)
      t0 := cnt + clkfreq / 100
      t1 := t0 + clkfreq
      waitcnt(t0)
      p0 += phsa
      p2 += phsb
      waitcnt(t1)
      p1 += phsa
      p3 += phsb
      'weight := p3 - p2 + p1 - p0
      weight := u.multdiv(1_000_000, p1 - p0, p3 - p2 + p1 - p0)     

DAT

'' Generate two square waves at the receive frequency, 90 degrees out of phase.

              org       0
              
quadrature    mov       dira,q_dira0            'Set the quadrature pins to output.
              
:query_lp     rdlong    freq,par                'Get the target receive frequency.
              cmp       freq,pfreq wz           'Same as previous reading?
        if_z  jmp       #:query_lp              '  Yes: Nothing to do.

              mov       ra,freq                 'Save frequency in high part of ra:rb.
              mov       rb,#0
              mov       rx,#%0_00100_000        'Assume NCO output at first.
              mov       _90deg,_4000_0000       'Shift for NCO is 2**32 / 4
              cmp       ra,_4mhz wc             'Is frequency less than 4MHz?
        if_c  jmp       #:do_mode               '  Yes: Can't do PLL, so continue w/NCO.

:do_pll       mov       rx,#%0_00010_011        '  No:  Change mode to PLLx1.

:pll_lp       cmp       ra,_8mhz wc             'Is NCO frequency less than 8MHz?
        if_c  jmp       #:do_mode               '  Yes: Done adjusting.

              add       rx,#1                   'Bump PLL mode by one.
              shr       ra,#1 wc                'Divide NCO frequency by two.
              rcr       rb,#1
              shr       _90deg,#1               'Also divide quadrature adder by two.
              jmp       #:pll_lp                'Try again.

:do_mode      movi      q_ctra0,rx              'Save PLL mode in counter buffers.
              movi      q_ctrb0,rx        
                                                
              rdlong    rx,#0                   'Fetch clk_freq.
              call      #divabx                 'Divide into NCO freq (* 2**32)
              
              mov       ctra,q_ctra0            'Set new counter modes to counters.
              mov       ctrb,q_ctrb0
              mov       frqa,#0                 'Stop counting momentarily.
              mov       frqb,#0
              mov       phsa,#0                 'Initialize one phase to zero,
              mov       phsb,_90deg             '  the other to the quadrature offset.
              add       phsb,rb                 'Then adjust the phase by 4xNCO freq
              add       phsb,rb                 '  to account for frqb being loaded  
              add       phsb,rb                 '  four clocks later than frqa.
              add       phsb,rb
              mov       frqa,rb                 'Now load the NCO frequencies.
              mov       frqb,rb
              mov       pfreq,freq              'Save the new frequency in pfreq.
              jmp       #:query_lp              'Done: back for a new frequency.

'-----------[ Divide 64 / 32 = 32 ]--------------------------------------------  

' rb := ra:rb / rx

divabx        mov       cctr,#32 wc
:loop         rcl       rb,#1 wc
              rcl       ra,#1 wc
        if_c  sub       ra,rx
       if_nc  cmpsub    ra,rx wc
              djnz      cctr,#:loop
              rcl       rb,#1 wc
divabx_ret    ret

'-----------[ Constants & Variables ]------------------------------------------


q_ctra0       long      %00100 << 26 | IF_PIN
q_ctrb0       long      %00100 << 26 | QF_PIN
q_dira0       long      1 << IF_PIN | 1 << QF_PIN

freq          long      0
pfreq         long      0

_4000_0000    long      $4000_0000
_4mhz         long      4_000_000
_8mhz         long      8_000_000

ra            res       1
rb            res       1
rx            res       1
cctr          res       1
_90deg        res       1

''-----------[ TERMS OF USE ]---------------------------------------------------
''
'' Permission is hereby granted, free of charge, to any person obtaining a copy of
'' this software and associated documentation files (the "Software"), to deal in
'' the Software without restriction, including without limitation the rights to use,
'' copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
'' Software, and to permit persons to whom the Software is furnished to do so,
'' subject to the following conditions: 
''
'' The above copyright notice and this permission notice shall be included in all
'' copies or substantial portions of the Software. 
''
'' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
'' INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
'' PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
'' HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
'' OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
'' SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


